"""Ultima Stock News Fetcher."""


from datetime import datetime
from typing import Any, Dict, List, Optional

from openbb_provider.abstract.fetcher import Fetcher
from openbb_provider.standard_models.stock_news import (
    StockNewsData,
    StockNewsQueryParams,
)
from openbb_ultima.utils.helpers import get_data
from pydantic import Field


class UltimaStockNewsQueryParams(StockNewsQueryParams):
    """Ultima Stock News query.

    Source: https://api.ultimainsights.ai/v1/api-docs#/default/get_v1_getOpenBBProInsights__tickers_
    """

    __alias_dict__ = {
        "symbols": "tickers",
    }


class UltimaStockNewsData(StockNewsData):
    """Ultima Stock News Data."""

    __alias_dict__ = {"date": "publishedDate", "text": "summary", "title": "headline"}

    publisher: str = Field(description="Publisher of the news.")
    ticker: str = Field(description="Ticker associated with the news.")
    riskCategory: str = Field(description="Risk category of the news.")


class UltimaStockNewsFetcher(
    Fetcher[
        UltimaStockNewsQueryParams,
        List[UltimaStockNewsData],
    ]
):
    @staticmethod
    def transform_query(params: Dict[str, Any]) -> UltimaStockNewsQueryParams:
        return UltimaStockNewsQueryParams(**params)

    @staticmethod
    def extract_data(
        query: UltimaStockNewsQueryParams,
        credentials: Optional[Dict[str, str]],
        **kwargs: Any,
    ) -> Dict:
        token = credentials.get("ultima_api_key") if credentials else ""
        kwargs["auth"] = token

        base_url = "https://api.ultimainsights.ai/v1/getOpenBBProInsights"

        querystring = str(query).split("=")[1].split("'")[1].replace(" ", "")

        data = []
        url = f"{base_url}/{querystring}"
        response = get_data(url, **kwargs)
        data.extend(response)

        return data

    @staticmethod
    def transform_data(
        query: UltimaStockNewsQueryParams,
        data: List[Dict],
        **kwargs: Any,
    ) -> List[UltimaStockNewsData]:
        results = []
        for ele in data:
            for key in ["8k_filings", "articles", "industry_summary"]:
                for item in ele[key]:
                    # manual assignment required for Pydantic to work
                    item["ticker"] = ele["ticker"]
                    item["date"] = datetime.strptime(
                        item["publishedDate"], "%Y-%m-%d %H:%M:%S"
                    )
                    item["title"] = item["headline"]
                    item["url"] = item["url"]
                    item["publisher"] = item["publisher"]
                    results.append(UltimaStockNewsData.model_validate(item))
        return results



"""Ultima provider module."""
from openbb_provider.abstract.provider import Provider
from openbb_ultima.models.stock_news import UltimaStockNewsFetcher

ultima_provider = Provider(
    name="ultima",
    website="https://www.ultimainsights.ai/openbb",
    description="""Ultima harnesses the power of LLMs to deliver news before it hits the frontpage of Bloomberg.""",
    required_credentials=["api_key"],
    fetcher_dict={
        "StockNews": UltimaStockNewsFetcher,
    },
)



"""Benzinga Helpers."""


from typing import Any, Dict

from openbb_provider import helpers


def get_data(url: str, **kwargs: Any) -> Dict:
    """Do an API request to Ultima and return the data."""
    auth = kwargs.pop("auth", "")
    if len(auth) == 0:
        raise RuntimeError("Ultima API key is required.")
    if "Bearer" not in auth:
        auth = f"Bearer {auth}"
    result = helpers.make_request(
        url,
        timeout=10,
        headers={"accept": "application/json", "Authorization": auth},
        **kwargs,
    )
    if result.status_code != 200:
        data = result.json()
        message = data.get("message")
        raise RuntimeError(f"Error in Ultima request -> {message}")

    return result.json()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['openbb_ultima', 'openbb_ultima.models', 'openbb_ultima.utils']

package_data = \
{'': ['*']}

install_requires = \
['openbb-core>=0.1.0a2,<0.2.0']

entry_points = \
{'openbb_provider_extension': ['ultima = openbb_ultima:ultima_provider']}

setup_kwargs = {
    'name': 'openbb-ultima',
    'version': '0.1.0a5',
    'description': 'Ultima Insights extension for OpenBB',
    'long_description': '# OpenBB Ultima Provider\n\nThis extension integrates the [Ultima Insights](https://www.ultimainsights.ai/) data provider into the OpenBB Platform.\n\n## Installation\n\nTo install the extension:\n\n```bash\npip install openbb-ultima\n```\n\nFor development please check [Contribution Guidelines](https://github.com/OpenBB-finance/OpenBBTerminal/blob/feature/openbb-sdk-v4/openbb_platform/CONTRIBUTING.md).\n\nDocumentation available [here](https://docs.openbb.co/sdk).\n',
    'author': 'Ultima Insights Team',
    'author_email': 'hello@ultimainsights.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
